
/*======================================================================
 * dirent directory routines for windows
 *======================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "dirent.h"

/*---------- opendir ---------------------------------------------------
 * opens a directory for reading
 *----------------------------------------------------------------------*/

DIR *opendir (char *dirname)
{
    char path[256];
    int count;
    struct stat dir_stat;
    long attrib;
    struct _finddata_t file_data;
    DIR *dirp;
    struct dirent *dp;

    if (dirname == NULL || !*dirname || !strcmp (dirname, "."))
        strcpy (path, "*.*");
    else {
        if (strlen (dirname) >= sizeof(path) - 5) {
            fprintf (stderr, "directory name is too long\n");
            return NULL;
        }
        strcpy (path, dirname);
        count = strlen (path);
        if (count == 1 && (*path == '/' || *path == '\\'))
            strcpy (path, "\\*.*");
        else {
            if (path[--count] == '\\' || path[count] == '/')
                path[count] = 0;
            if (stat (path, &dir_stat) ||
                (dir_stat.st_mode & S_IFMT) != S_IFDIR)
                return NULL;
            strcat (path, "\\*.*");
        }
    }

    /* scan directory and get number of entries */

    if ((attrib = _findfirst (path, &file_data)) < 0)
        return NULL;
    count = 1;
    while (_findnext (attrib, &file_data) == 0)
        count++;

    /* malloc space for structures */

    if ((dirp = (DIR *) malloc (sizeof(DIR))) == NULL) {
        fprintf (stderr, "malloc failed for DIR structure\n");
        return NULL;
    }
    dirp->dd_size = count * sizeof(struct dirent);
    if ((dp = (struct dirent *) malloc (dirp->dd_size)) == NULL) {
        fprintf (stderr, "malloc failed for directory list\n");
        free (dirp);
        return NULL;
    }

    dirp->dd_buf = (char *)dp;
    dirp->dd_loc = 0;

    /* reread directory and fill in dirent structure */

    attrib = _findfirst (path, &file_data);
    count = 0;
    do {
        dp->d_ino = 1;
        dp->d_off = ++count;
        dp->d_reclen = sizeof(struct dirent);
        strcpy (dp->d_name, file_data.name);
        dp++;
    } while (_findnext (attrib, &file_data) == 0);
    return (dirp);
}

/*---------- readdir ---------------------------------------------------
 * gets next entry in directory structure
 *----------------------------------------------------------------------*/

struct dirent *readdir (DIR *dirp)
{
    struct dirent *dp;

    if (dirp == NULL || dirp->dd_loc < 0 || dirp->dd_loc >= dirp->dd_size)
        return NULL;
    dp = (struct dirent *)(dirp->dd_buf + dirp->dd_loc);
    dirp->dd_loc += sizeof(struct dirent);
    return (dp);
}

/*---------- telldir ---------------------------------------------------
 * returns position in directory structure
 *----------------------------------------------------------------------*/

long telldir (DIR *dirp)
{
    if (dirp == NULL)
        return (0L);
    return ((long)dirp->dd_loc);
}

/*---------- seekdir ---------------------------------------------------
 * seeks to position in directory structure
 *----------------------------------------------------------------------*/

void seekdir (DIR *dirp, long loc)
{
    if (dirp != NULL)
        dirp->dd_loc = (int)loc;
}

/*---------- closedir --------------------------------------------------
 * closes directory structure and frees memory
 *----------------------------------------------------------------------*/

void closedir (DIR *dirp)
{
    if (dirp == NULL) {
        free (dirp->dd_buf);
        free (dirp);
    }
}

